import rclpy
from rclpy.node import Node

from std_msgs.msg import Float32

from serial import Serial

from enum import Enum

class Servos(Enum):
    BODY = 0
    ARM = 1
    HAND = 2

class CobotMove(Node):
    def __init__(self):
        super().__init__('cobot_move')
        self._logger = self.get_logger()
        self._logger.info('COBOT init')

        self._serial = Serial('/dev/ttyACM0', 115200)

        # self._buf = [90, 90, 90] # 3 servos
        # self._prev_buf = [90, 90, 90]

        base_topic = 'move/'

        self.create_subscription(Float32, base_topic+'body', self._move_body_cb, rclpy.qos.qos_profile_sensor_data)
        self.create_subscription(Float32, base_topic+'arm',  self._move_arm_cb,  rclpy.qos.qos_profile_sensor_data)
        self.create_subscription(Float32, base_topic+'hand', self._move_hand_cb, rclpy.qos.qos_profile_sensor_data)

    def _move_body_cb(self, msg):
        self._move_servo(Servos.BODY, msg.data)

    def _move_arm_cb(self, msg):
        self._move_servo(Servos.ARM, msg.data)

    def _move_hand_cb(self, msg):
        self._move_servo(Servos.HAND, msg.data)

    def _move_servo(self, serv_num, pos):
        cmd = 'SV'
        #  self._buf[serv_num] = pos
        self._logger.info('COBOT: Servo {serv_num} to {pos}')
        self._serial.write(f'{cmd}V{serv_num}P{pos}\n'.encode('UTF-8'))

    # def _push ?

def main(args=None):
    rclpy.init(args=args)

    cobot_move = CobotMove()
    rclpy.spin(cobot_move)

    cobot_move.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
